
#ifndef TARTANAFILE_HH
#define TARTANAFILE_HH

#include <map>
#include <vector>
#include <string>

#include "TObject.h"

#include "EAnalyser.hh"
#include "EWNum.hh"

class TH1;

class TArtValIndex
{
public:  
  TArtValIndex(){;}
  TArtValIndex(EAnalyser analyser, EWNum wnum, int index)
    : fAnalyser(analyser), fWNum(wnum), fIndex(index){;}
  virtual ~TArtValIndex(){;}

  int fAnalyser, fWNum, fIndex;
 
  ClassDef(TArtValIndex,1);
};

class TArtVal
{
public:
  TArtVal(){;}
  TArtVal(TArtValIndex valindex, Double_t val)
    : fValIndex(valindex), fVal(val) {;}
  virtual ~TArtVal(){;}

  TArtValIndex fValIndex;
  Double_t fVal;

  ClassDef(TArtVal,1);
};

class TArtValArray : public std::vector<TArtVal>
{
public:
  TArtValArray(){;}
  virtual ~TArtValArray(){;}

  bool Add(TArtValIndex valindex, Double_t val);
  bool Find(TArtValIndex valindex);
  bool Find(TArtValIndex valindex, Double_t& val);
  bool Find(TArtValIndex valindexx, TArtValIndex valindexy,
	    Double_t& valx, Double_t& valy);

  ClassDef(TArtValArray,1);
};

class TArtAnaGate
{
public:
  TArtAnaGate()
    : fGate(0), fLow(0), fUp(-1), fInside(true), fAnd(false), fOr(false){;}
  TArtAnaGate(int gate, Double_t low, Double_t up)
    : fGate(gate), fLow(low), fUp(up), fInside(false), fAnd(false), fOr(false){;}
  TArtAnaGate(int gate, bool And, bool Or, const std::vector<int>& gatearray)
    : fGate(gate), fLow(0), fUp(-1), fInside(false), fAnd(And), fOr(Or), fGateArray(gatearray){;}
  virtual ~TArtAnaGate(){;}

  int fGate;
  Double_t fLow, fUp;
  bool fInside;

  bool fAnd, fOr;
  std::vector<int> fGateArray;

  ClassDef(TArtAnaGate,1);
};

class TArtAnaFile
{
public:

public:
  TArtAnaFile(){;}
  virtual ~TArtAnaFile(){;}

  bool Open(const char* filename);
  bool Check();
  void Clear();
  bool Add(EAnalyser analyser, EWNum wnum, int index, Double_t val);
  void Fill();

private:
  bool AddValIndexMap(std::string name, std::vector<TArtValIndex> v_valindex);
  bool AddGateMap(std::string name, int gate);
  bool AddValIndexMapForGate(int gate, std::vector<TArtValIndex> v_valindex);
  bool AddAnaGateMap(int gate, TArtAnaGate anagate);

  bool CalGate(int gate);
  bool IsInside(int gate);

  TArtValArray fValArray;

  std::vector<TH1*> fH1Array;
  std::map<std::string, std::vector<TArtValIndex> > fValIndexMap;
  std::map<std::string, int> fGateMap;

  std::map<int, TArtAnaGate> fAnaGateMap;
  std::map<int, std::vector<TArtValIndex> > fValIndexMapForGate;

  ClassDef(TArtAnaFile,1);
};

#endif
